@@ -1,4 +1,4 @@
-# Copyright 2017-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -469,6 +469,9 @@ class LocalSagemakerRuntimeClient(object):
         if InferenceId is not None:
             headers["X-Amzn-SageMaker-Inference-Id"] = InferenceId
 
+        # The http client encodes all strings using latin-1, which is not what we want.
+        if isinstance(Body, str):
+            Body = Body.encode("utf-8")
         r = self.http.request("POST", url, body=Body, preload_content=False, headers=headers)
 
         return {"Body": r, "ContentType": Accept}
