@@ -1,4 +1,4 @@
-# Copyright 2017-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -13,6 +13,7 @@
 """Placeholder docstring"""
 from __future__ import absolute_import
 
+import abc
 import json
 import logging
 import os
@@ -29,6 +30,7 @@ from sagemaker import (
     git_utils,
 )
 from sagemaker.deprecations import removed_kwargs
+from sagemaker.predictor import PredictorBase
 from sagemaker.transformer import Transformer
 
 LOGGER = logging.getLogger("sagemaker")
@@ -38,7 +40,23 @@ NEO_ALLOWED_FRAMEWORKS = set(
 )
 
 
-class Model(object):
+class ModelBase(abc.ABC):
+    """An object that encapsulates a trained model.
+
+    Models can be deployed to compute services like a SageMaker ``Endpoint``
+    or Lambda. Deployed models can be used to perform real-time inference.
+    """
+
+    @abc.abstractmethod
+    def deploy(self, *args, **kwargs) -> PredictorBase:
+        """Deploy this model to a compute service."""
+
+    @abc.abstractmethod
+    def delete_model(self, *args, **kwargs) -> None:
+        """Destroy resources associated with this model."""
+
+
+class Model(ModelBase):
     """A SageMaker ``Model`` that can be deployed to an ``Endpoint``."""
 
     def __init__(
@@ -158,14 +176,15 @@ class Model(object):
         if self.model_data is None:
             raise ValueError("SageMaker Model Package cannot be created without model data.")
 
-        model_pkg_args = self._get_model_package_args(
+        model_pkg_args = sagemaker.get_model_package_args(
             content_types,
             response_types,
             inference_instances,
             transform_instances,
             model_package_name,
             model_package_group_name,
-            image_uri,
+            self.model_data,
+            image_uri or self.image_uri,
             model_metrics,
             metadata_properties,
             marketplace_cert,
@@ -181,77 +200,6 @@ class Model(object):
             model_package_arn=model_package.get("ModelPackageArn"),
         )
 
-    def _get_model_package_args(
-        self,
-        content_types,
-        response_types,
-        inference_instances,
-        transform_instances,
-        model_package_name=None,
-        model_package_group_name=None,
-        image_uri=None,
-        model_metrics=None,
-        metadata_properties=None,
-        marketplace_cert=False,
-        approval_status=None,
-        description=None,
-    ):
-        """Get arguments for session.create_model_package method.
-
-        Args:
-            content_types (list): The supported MIME types for the input data.
-            response_types (list): The supported MIME types for the output data.
-            inference_instances (list): A list of the instance types that are used to
-                generate inferences in real-time.
-            transform_instances (list): A list of the instance types on which a transformation
-                job can be run or on which an endpoint can be deployed.
-            model_package_name (str): Model Package name, exclusive to `model_package_group_name`,
-                using `model_package_name` makes the Model Package un-versioned (default: None).
-            model_package_group_name (str): Model Package Group name, exclusive to
-                `model_package_name`, using `model_package_group_name` makes the Model Package
-                versioned (default: None).
-            image_uri (str): Inference image uri for the container. Model class' self.image will
-                be used if it is None (default: None).
-            model_metrics (ModelMetrics): ModelMetrics object (default: None).
-            metadata_properties (MetadataProperties): MetadataProperties object (default: None).
-            marketplace_cert (bool): A boolean value indicating if the Model Package is certified
-                for AWS Marketplace (default: False).
-            approval_status (str): Model Approval Status, values can be "Approved", "Rejected",
-                or "PendingManualApproval" (default: "PendingManualApproval").
-            description (str): Model Package description (default: None).
-        Returns:
-            dict: A dictionary of method argument names and values.
-        """
-        if image_uri:
-            self.image_uri = image_uri
-        container = {
-            "Image": self.image_uri,
-            "ModelDataUrl": self.model_data,
-        }
-
-        model_package_args = {
-            "containers": [container],
-            "content_types": content_types,
-            "response_types": response_types,
-            "inference_instances": inference_instances,
-            "transform_instances": transform_instances,
-            "marketplace_cert": marketplace_cert,
-        }
-
-        if model_package_name is not None:
-            model_package_args["model_package_name"] = model_package_name
-        if model_package_group_name is not None:
-            model_package_args["model_package_group_name"] = model_package_group_name
-        if model_metrics is not None:
-            model_package_args["model_metrics"] = model_metrics._to_request_dict()
-        if metadata_properties is not None:
-            model_package_args["metadata_properties"] = metadata_properties._to_request_dict()
-        if approval_status is not None:
-            model_package_args["approval_status"] = approval_status
-        if description is not None:
-            model_package_args["description"] = description
-        return model_package_args
-
     def _init_sagemaker_session_if_does_not_exist(self, instance_type):
         """Set ``self.sagemaker_session`` to ``LocalSession`` or ``Session`` if it's not already.
 
@@ -556,7 +504,7 @@ class Model(object):
         role,
         tags=None,
         job_name=None,
-        compile_max_run=5 * 60,
+        compile_max_run=15 * 60,
         framework=None,
         framework_version=None,
         target_platform_os=None,
@@ -585,7 +533,7 @@ class Model(object):
                 https://docs.aws.amazon.com/sagemaker/latest/dg/API_Tag.html.
             job_name (str): The name of the compilation job
             compile_max_run (int): Timeout in seconds for compilation (default:
-                3 * 60). After this amount of time Amazon SageMaker Neo
+                15 * 60). After this amount of time Amazon SageMaker Neo
                 terminates the compilation job regardless of its current status.
             framework (str): The framework that is used to train the original
                 model. Allowed values: 'mxnet', 'tensorflow', 'keras', 'pytorch',
@@ -1111,7 +1059,7 @@ class FrameworkModel(Model):
     def _upload_code(self, key_prefix, repack=False):
         """Placeholder Docstring"""
         local_code = utils.get_config_value("local.local_code", self.sagemaker_session.config)
-        if self.sagemaker_session.local_mode and local_code:
+        if (self.sagemaker_session.local_mode and local_code) or self.entry_point is None:
             self.uploaded_code = None
         elif not repack:
             bucket = self.bucket or self.sagemaker_session.default_bucket()
@@ -1124,7 +1072,11 @@ class FrameworkModel(Model):
                 dependencies=self.dependencies,
             )
 
-        if repack:
+        if repack and self.model_data is not None and self.entry_point is not None:
+            if isinstance(self.model_data, sagemaker.workflow.properties.Properties):
+                # model is not yet there, defer repacking to later during pipeline execution
+                return
+
             bucket = self.bucket or self.sagemaker_session.default_bucket()
             repacked_model_data = "s3://" + "/".join([bucket, key_prefix, "model.tar.gz"])
 
@@ -1159,8 +1111,8 @@ class FrameworkModel(Model):
             dir_name = None
 
         return {
-            SCRIPT_PARAM_NAME.upper(): script_name,
-            DIR_PARAM_NAME.upper(): dir_name,
+            SCRIPT_PARAM_NAME.upper(): script_name or str(),
+            DIR_PARAM_NAME.upper(): dir_name or str(),
             CONTAINER_LOG_LEVEL_PARAM_NAME.upper(): str(self.container_log_level),
             SAGEMAKER_REGION_PARAM_NAME.upper(): self.sagemaker_session.boto_region_name,
         }
