@@ -1,4 +1,4 @@
-# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -17,7 +17,6 @@ import os
 import shutil
 import tarfile
 import tempfile
-
 from typing import List
 
 from sagemaker import image_uris
@@ -30,6 +29,8 @@ from sagemaker.estimator import EstimatorBase
 from sagemaker.sklearn.estimator import SKLearn
 from sagemaker.workflow.entities import RequestType
 from sagemaker.workflow.properties import Properties
+from sagemaker.session import get_create_model_package_request
+from sagemaker.session import get_model_package_args
 from sagemaker.workflow.steps import (
     StepTypeEnum,
     TrainingStep,
@@ -54,12 +55,14 @@ class _RepackModelStep(TrainingStep):
     def __init__(
         self,
         name: str,
-        estimator: EstimatorBase,
+        sagemaker_session,
+        role,
         model_data: str,
         entry_point: str,
         source_dir: str = None,
         dependencies: List = None,
         depends_on: List[str] = None,
+        **kwargs,
     ):
         """Constructs a TrainingStep, given an `EstimatorBase` instance.
 
@@ -72,8 +75,9 @@ class _RepackModelStep(TrainingStep):
             inputs (TrainingInput): A `sagemaker.inputs.TrainingInput` instance. Defaults to `None`.
         """
         # yeah, go ahead and save the originals for now
-        self._estimator = estimator
         self._model_data = model_data
+        self.sagemaker_session = sagemaker_session
+        self.role = role
         if isinstance(model_data, Properties):
             self._model_prefix = model_data
             self._model_archive = "model.tar.gz"
@@ -92,12 +96,13 @@ class _RepackModelStep(TrainingStep):
             entry_point=REPACK_SCRIPT,
             source_dir=self._source_dir,
             dependencies=self._dependencies,
-            sagemaker_session=self._estimator.sagemaker_session,
-            role=self._estimator.role,
+            sagemaker_session=self.sagemaker_session,
+            role=self.role,
             hyperparameters={
                 "inference_script": self._entry_point_basename,
                 "model_archive": self._model_archive,
             },
+            **kwargs,
         )
         repacker.disable_profiler = True
         inputs = TrainingInput(self._model_prefix)
@@ -148,7 +153,7 @@ class _RepackModelStep(TrainingStep):
                 S3Downloader.download(
                     s3_uri=self._source_dir,
                     local_path=local_path,
-                    sagemaker_session=self._estimator.sagemaker_session,
+                    sagemaker_session=self.sagemaker_session,
                 )
 
                 src_dir = os.path.join(tmp, "src")
@@ -162,7 +167,7 @@ class _RepackModelStep(TrainingStep):
                 S3Uploader.upload(
                     local_path=local_path,
                     desired_s3_uri=self._source_dir,
-                    sagemaker_session=self._estimator.sagemaker_session,
+                    sagemaker_session=self.sagemaker_session,
                 )
         else:
             shutil.copy2(fname, os.path.join(self._source_dir, REPACK_SCRIPT))
@@ -205,18 +210,19 @@ class _RegisterModelStep(Step):
             Estimator's training container image will be used (default: None).
         compile_model_family (str): Instance family for compiled model, if specified, a compiled
             model will be used (default: None).
+        container_def_list (list): A list of container definitions.
         **kwargs: additional arguments to `create_model`.
     """
 
     def __init__(
         self,
         name: str,
-        estimator: EstimatorBase,
-        model_data,
         content_types,
         response_types,
         inference_instances,
         transform_instances,
+        estimator: EstimatorBase = None,
+        model_data=None,
         model_package_group_name=None,
         model_metrics=None,
         metadata_properties=None,
@@ -225,6 +231,8 @@ class _RegisterModelStep(Step):
         compile_model_family=None,
         description=None,
         depends_on: List[str] = None,
+        tags=None,
+        container_def_list=None,
         **kwargs,
     ):
         """Constructor of a register model step.
@@ -254,6 +262,7 @@ class _RegisterModelStep(Step):
             description (str): Model Package description (default: None).
             depends_on (List[str]): A list of step names this `sagemaker.workflow.steps.TrainingStep`
                 depends on
+            container_def_list (list): A list of containers.
             **kwargs: additional arguments to `create_model`.
         """
         super(_RegisterModelStep, self).__init__(name, StepTypeEnum.REGISTER_MODEL, depends_on)
@@ -264,13 +273,16 @@ class _RegisterModelStep(Step):
         self.inference_instances = inference_instances
         self.transform_instances = transform_instances
         self.model_package_group_name = model_package_group_name
+        self.tags = tags
         self.model_metrics = model_metrics
         self.metadata_properties = metadata_properties
         self.approval_status = approval_status
         self.image_uri = image_uri
         self.compile_model_family = compile_model_family
         self.description = description
+        self.tags = tags
         self.kwargs = kwargs
+        self.container_def_list = container_def_list
 
         self._properties = Properties(
             path=f"Steps.{name}", shape_name="DescribeModelPackageResponse"
@@ -280,54 +292,66 @@ class _RegisterModelStep(Step):
     def arguments(self) -> RequestType:
         """The arguments dict that are used to call `create_model_package`."""
         model_name = self.name
-        if self.compile_model_family:
-            model = self.estimator._compiled_models[self.compile_model_family]
-        else:
-            # create_model wants the estimator to have a model_data attribute...
-            self.estimator._current_job_name = model_name
 
-            # placeholder. replaced with model_data later
-            output_path = self.estimator.output_path
-            self.estimator.output_path = "/tmp"
-
-            # create the model, but custom funky framework stuff going on in some places
-            if self.image_uri:
-                model = self.estimator.create_model(image_uri=self.image_uri, **self.kwargs)
+        if self.container_def_list is None:
+            if self.compile_model_family:
+                model = self.estimator._compiled_models[self.compile_model_family]
+                self.model_data = model.model_data
             else:
-                model = self.estimator.create_model(**self.kwargs)
-            model.model_data = self.model_data
+                # create_model wants the estimator to have a model_data attribute...
+                self.estimator._current_job_name = model_name
 
-            # reset placeholder
-            self.estimator.output_path = output_path
+                # placeholder. replaced with model_data later
+                output_path = self.estimator.output_path
+                self.estimator.output_path = "/tmp"
 
-            # yeah, there is some framework stuff going on that we need to pull in here
-            if model.image_uri is None:
-                region_name = self.estimator.sagemaker_session.boto_session.region_name
-                model.image_uri = image_uris.retrieve(
-                    model._framework_name,
-                    region_name,
-                    version=model.framework_version,
-                    py_version=model.py_version,
-                    instance_type=self.kwargs.get("instance_type", self.estimator.instance_type),
-                    accelerator_type=self.kwargs.get("accelerator_type"),
-                    image_scope="inference",
-                )
-        model.name = model_name
+                # create the model, but custom funky framework stuff going on in some places
+                if self.image_uri:
+                    model = self.estimator.create_model(image_uri=self.image_uri, **self.kwargs)
+                else:
+                    model = self.estimator.create_model(**self.kwargs)
+                    self.image_uri = model.image_uri
 
-        model_package_args = model._get_model_package_args(
+                if self.model_data is None:
+                    self.model_data = model.model_data
+
+                # reset placeholder
+                self.estimator.output_path = output_path
+
+                # yeah, there is some framework stuff going on that we need to pull in here
+                if self.image_uri is None:
+                    region_name = self.estimator.sagemaker_session.boto_session.region_name
+                    self.image_uri = image_uris.retrieve(
+                        model._framework_name,
+                        region_name,
+                        version=model.framework_version,
+                        py_version=model.py_version if hasattr(model, "py_version") else None,
+                        instance_type=self.kwargs.get(
+                            "instance_type", self.estimator.instance_type
+                        ),
+                        accelerator_type=self.kwargs.get("accelerator_type"),
+                        image_scope="inference",
+                    )
+                    model.name = model_name
+                    model.model_data = self.model_data
+
+        model_package_args = get_model_package_args(
             content_types=self.content_types,
             response_types=self.response_types,
             inference_instances=self.inference_instances,
             transform_instances=self.transform_instances,
             model_package_group_name=self.model_package_group_name,
+            model_data=self.model_data,
+            image_uri=self.image_uri,
             model_metrics=self.model_metrics,
             metadata_properties=self.metadata_properties,
             approval_status=self.approval_status,
             description=self.description,
+            tags=self.tags,
+            container_def_list=self.container_def_list,
         )
-        request_dict = model.sagemaker_session._get_create_model_package_request(
-            **model_package_args
-        )
+
+        request_dict = get_create_model_package_request(**model_package_args)
         # these are not available in the workflow service and will cause rejection
         if "CertifyForMarketplace" in request_dict:
             request_dict.pop("CertifyForMarketplace")
